@@ -30,13 +30,13 @@ import gi
 gi.require_versions({"Grl": "0.3", "Tracker": "3.0"})
 from gi.repository import Grl, Gio, Gtk, GLib, GObject, Tracker
 
-from gnomemusic.asyncqueue import AsyncQueue
 from gnomemusic.corealbum import CoreAlbum
 from gnomemusic.coreartist import CoreArtist
 from gnomemusic.coredisc import CoreDisc
 from gnomemusic.coresong import CoreSong
 from gnomemusic.grilowrappers.grltrackerplaylists import (
     GrlTrackerPlaylists, Playlist)
+from gnomemusic.storeart import StoreArt
 from gnomemusic.trackerwrapper import TrackerWrapper
 from gnomemusic.utils import CoreObjectType
 if typing.TYPE_CHECKING:
@@ -44,7 +44,6 @@ if typing.TYPE_CHECKING:
     from gnomemusic.coremodel import CoreModel
     from gnomemusic.musiclogger import MusicLogger
     from gnomemusic.notificationmanager import NotificationManager
-    from gnomemusic.storeart import StoreArt
 
 
 class GrlTrackerWrapper(GObject.GObject):
@@ -105,7 +104,6 @@ class GrlTrackerWrapper(GObject.GObject):
         """
         super().__init__()
 
-        self._async_queue = AsyncQueue("GrlTrackerWrapper")
         self._application: Application = application
         cm: CoreModel = application.props.coremodel
         self._log: MusicLogger = application.props.log
@@ -163,6 +161,8 @@ class GrlTrackerWrapper(GObject.GObject):
             Grl.ResolutionFlags.FULL | Grl.ResolutionFlags.IDLE_RELAY)
 
         self._content_changed_id: int = 0
+        self._grilo_search_operation_ids: List[int] = []
+
         self.props.source = source
 
         self._initial_songs_fill()
@@ -889,17 +889,8 @@ class GrlTrackerWrapper(GObject.GObject):
         self.props.source.query(
             query, metadata_keys, self._fast_options, _callback)
 
-    def search(self, text: str) -> None:
-        # FIXME: Searches are limited to not bog down the UI with
-        # widget creation ({List,Flow}Box limitations). The limit is
-        # arbitrarily set to 50 and set in the Tracker query. It should
-        # be possible to set it through Grilo options instead. This
-        # does not work as expected and needs further investigation.
-        term: str = Tracker.sparql_escape_string(
-            GLib.utf8_normalize(
-                GLib.utf8_casefold(text, -1), -1, GLib.NormalizeMode.NFKD))
-
-        # Artist search
+    def _search_artist(self, term: str) -> None:
+        """Search the artist tag and display results.."""
         self._notificationmanager.push_loading()
 
         query = """
@@ -953,32 +944,10 @@ class GrlTrackerWrapper(GObject.GObject):
             'name': term
         }
 
-        artist_filter_ids: List[str] = []
+        self._run_query(query, self._artists_search)
 
-        def artist_filter(coreartist: CoreArtist) -> bool:
-            return coreartist.media.get_id() in artist_filter_ids
-
-        def artist_search_cb(
-                source: Grl.Source, op_id: int, media: Optional[Grl.Media],
-                remaining: int, error: Optional[GLib.Error]) -> None:
-            if error:
-                self._log.warning("Error: {}".format(error))
-                self._notificationmanager.pop_loading()
-                return
-
-            if not media:
-                custom_filter = Gtk.CustomFilter()
-                custom_filter.set_filter_func(artist_filter)
-                self._artists_search.set_filter(custom_filter)
-                self._notificationmanager.pop_loading()
-                return
-
-            artist_filter_ids.append(media.get_id())
-
-        self.props.source.query(
-            query, [Grl.METADATA_KEY_ID], self._fast_options, artist_search_cb)
-
-        # Album search
+    def _search_album(self, term: str) -> None:
+        """Search the album tag and display results."""
         self._notificationmanager.push_loading()
 
         query = """
@@ -1024,32 +993,10 @@ class GrlTrackerWrapper(GObject.GObject):
             'name': term
         }
 
-        album_filter_ids: List[str] = []
+        self._run_query(query, self._albums_search)
 
-        def album_filter(corealbum: CoreAlbum) -> bool:
-            return corealbum.media.get_id() in album_filter_ids
-
-        def albums_search_cb(
-                source: Grl.Source, op_id: int, media: Optional[Grl.Media],
-                remaining: int, error: Optional[GLib.Error]) -> None:
-            if error:
-                self._log.warning("Error: {}".format(error))
-                self._notificationmanager.pop_loading()
-                return
-
-            if not media:
-                custom_filter = Gtk.CustomFilter()
-                custom_filter.set_filter_func(album_filter)
-                self._albums_search.set_filter(custom_filter)
-                self._notificationmanager.pop_loading()
-                return
-
-            album_filter_ids.append(media.get_id())
-
-        self.props.source.query(
-            query, [Grl.METADATA_KEY_ID], self._fast_options, albums_search_cb)
-
-        # Song search
+    def _search_song(self, term: str) -> None:
+        """Search for song names and display results."""
         self._notificationmanager.push_loading()
 
         query = """
@@ -1100,30 +1047,68 @@ class GrlTrackerWrapper(GObject.GObject):
             'name': term
         }
 
+        self._run_query(query, self._songs_search)
+
+    def _run_query(
+            self, query: str, filter_list_model: Gtk.FilterListModel) -> None:
+        """Run a SPARQL query and display results."""
         filter_ids: List[str] = []
 
-        def songs_filter(coresong: CoreSong) -> bool:
-            return coresong.media.get_id() in filter_ids
+        def filter_func(obj: GObject.GObject) -> bool:
+            return obj.media.get_id() in filter_ids
 
-        def songs_search_cb(
+        def _search_callback(
                 source: Grl.Source, op_id: int, media: Optional[Grl.Media],
                 remaining: int, error: Optional[GLib.Error]) -> None:
             if error:
-                self._log.warning("Error: {}".format(error))
+                if error.code != Grl.CoreError.OPERATION_CANCELLED:
+                    self._log.warning("Error: {}".format(error))
+                    self._albums_search.set_filter(Gtk.AnyFilter())
+                    self._grilo_search_operation_ids.remove(op_id)
                 self._notificationmanager.pop_loading()
                 return
 
             if not media:
                 custom_filter = Gtk.CustomFilter()
-                custom_filter.set_filter_func(songs_filter)
-                self._songs_search.set_filter(custom_filter)
+                custom_filter.set_filter_func(filter_func)
+                filter_list_model.set_filter(custom_filter)
                 self._notificationmanager.pop_loading()
+                self._grilo_search_operation_ids.remove(op_id)
                 return
 
             filter_ids.append(media.get_id())
 
-        self.props.source.query(
-            query, [Grl.METADATA_KEY_ID], self._fast_options, songs_search_cb)
+        op_id = self.props.source.query(
+            query, [Grl.METADATA_KEY_ID], self._fast_options, _search_callback)
+        self._grilo_search_operation_ids.append(op_id)
+
+    def search(self, text: str) -> None:
+        """Do a search in all relevant tags and display results.
+
+        :param str text: An arbitrary user-defined search string.
+        """
+        # FIXME: Searches are limited to not bog down the UI with
+        # widget creation ({List,Flow}Box limitations). The limit is
+        # arbitrarily set to 50 and set in the Tracker query. It should
+        # be possible to set it through Grilo options instead. This
+        # does not work as expected and needs further investigation.
+        term: str = Tracker.sparql_escape_string(
+            GLib.utf8_normalize(
+                GLib.utf8_casefold(text, -1), -1, GLib.NormalizeMode.NFKD))
+
+        for operation_id in self._grilo_search_operation_ids:
+            Grl.operation_cancel(operation_id)
+            self._grilo_search_operation_ids.remove(operation_id)
+
+        if text == "":
+            self._artists_search.set_filter(Gtk.AnyFilter())
+            self._albums_search.set_filter(Gtk.AnyFilter())
+            self._songs_search.set_filter(Gtk.AnyFilter())
+            return
+
+        self._search_artist(term)
+        self._search_album(term)
+        self._search_song(term)
 
     def _get_album_for_media_id_query(
             self, media_id: str, song: bool = True) -> str:
@@ -1186,7 +1171,7 @@ class GrlTrackerWrapper(GObject.GObject):
 
         return query
 
-    def get_song_art(self, coresong: CoreSong, storeart: StoreArt) -> None:
+    def get_song_art(self, coresong: CoreSong) -> None:
         """Retrieve song art for the given CoreSong
 
         Since MediaArt does not really support per-song art this
@@ -1194,7 +1179,6 @@ class GrlTrackerWrapper(GObject.GObject):
         art and store it.
 
         :param CoreSong coresong: CoreSong to get art for
-        :param StoreArt storeart: StoreArt instance to use
         """
         media: Grl.Media = coresong.props.media
 
@@ -1211,15 +1195,13 @@ class GrlTrackerWrapper(GObject.GObject):
                 error: Optional[GLib.Error]) -> None:
             if error:
                 self._log.warning("Error: {}".format(error))
-                self.emit("finished")
                 return
 
             if queried_media is None:
                 return
 
-            self._async_queue.queue(
-                storeart, coresong, queried_media.get_thumbnail(),
-                CoreObjectType.SONG)
+            StoreArt().start(
+                coresong, queried_media.get_thumbnail(), CoreObjectType.SONG)
 
         song_id: str = media.get_id()
         query: str = self._get_album_for_media_id_query(song_id)
@@ -1228,11 +1210,10 @@ class GrlTrackerWrapper(GObject.GObject):
             query, self._METADATA_THUMBNAIL_KEYS, self._full_options_lprio,
             art_retrieved_cb)
 
-    def get_album_art(self, corealbum: CoreAlbum, storeart: StoreArt) -> None:
+    def get_album_art(self, corealbum: CoreAlbum) -> None:
         """Retrieve album art for the given CoreAlbum
 
         :param CoreAlbum corealbum: CoreAlbum to get art for
-        :param StoreArt storeart: StoreArt instance to use
         """
         media: Grl.Media = corealbum.props.media
 
@@ -1242,15 +1223,13 @@ class GrlTrackerWrapper(GObject.GObject):
                 error: Optional[GLib.Error]) -> None:
             if error:
                 self._log.warning("Error: {}".format(error))
-                storeart.emit("finished")
                 return
 
             if queried_media is None:
                 return
 
-            self._async_queue.queue(
-                storeart, corealbum, queried_media.get_thumbnail(),
-                CoreObjectType.ALBUM)
+            StoreArt().start(
+                corealbum, queried_media.get_thumbnail(), CoreObjectType.ALBUM)
 
         album_id: str = media.get_id()
         query: str = self._get_album_for_media_id_query(album_id, False)
@@ -1259,14 +1238,12 @@ class GrlTrackerWrapper(GObject.GObject):
             query, self._METADATA_THUMBNAIL_KEYS, self._full_options_lprio,
             art_retrieved_cb)
 
-    def get_artist_art(
-            self, coreartist: CoreArtist, storeart: StoreArt) -> None:
+    def get_artist_art(self, coreartist: CoreArtist) -> None:
         """Retrieve artist art for the given CoreArtist
 
         This retrieves art through Grilo online services only.
 
         :param CoreArtist coreartist: CoreArtist to get art for
-        :param StoreArt storeart: StoreArt instance to use
         """
         media: Grl.Media = coreartist.props.media
 
@@ -1275,22 +1252,17 @@ class GrlTrackerWrapper(GObject.GObject):
                 resolved_media: Optional[Grl.Media],
                 error: Optional[GLib.Error]) -> None:
             if error:
-                storeart.emit("finished")
                 self._log.warning("Error: {}".format(error))
                 return
 
             if resolved_media is None:
-                storeart.emit("finished")
                 return
 
             thumbnail = resolved_media.get_thumbnail()
             if thumbnail is None:
-                storeart.emit("finished")
                 return
 
-            self._async_queue.queue(
-                storeart, coreartist, resolved_media.get_thumbnail(),
-                CoreObjectType.ARTIST)
+            StoreArt().start(coreartist, thumbnail, CoreObjectType.ARTIST)
 
         self.props.source.resolve(
             media, [Grl.METADATA_KEY_THUMBNAIL], self._full_options_lprio,
@@ -1307,7 +1279,7 @@ class GrlTrackerWrapper(GObject.GObject):
         self._tracker_playlists.stage_playlist_deletion(playlist)
 
     def finish_playlist_deletion(
-            self, playlist: Optional[Playlist], deleted: bool) -> None:
+            self, playlist: Playlist, deleted: bool) -> None:
         """Finishes playlist deletion.
 
         :param Playlist playlist: playlist
