@@ -23,7 +23,7 @@
 # delete this exception statement from your version.
 
 from __future__ import annotations
-from gettext import ngettext
+from gettext import gettext as _, ngettext
 from typing import Optional, Union
 import typing
 
@@ -61,7 +61,6 @@ class AlbumWidget(Adw.Bin):
     _released_label = Gtk.Template.Child()
     _title_label = Gtk.Template.Child()
 
-    selection_mode = GObject.Property(type=bool, default=False)
     show_artist_label = GObject.Property(type=bool, default=True)
 
     def __init__(self, application: Application) -> None:
@@ -88,17 +87,10 @@ class AlbumWidget(Adw.Bin):
             self, ArtSize.LARGE, DefaultIconType.ALBUM)
         self._player = self._application.props.player
 
-        self.bind_property(
-            "selection-mode", self._disc_list_box, "selection-mode",
-            GObject.BindingFlags.BIDIRECTIONAL
-            | GObject.BindingFlags.SYNC_CREATE)
-
         self.bind_property(
             "show-artist-label", self._artist_label, "visible",
             GObject.BindingFlags.SYNC_CREATE)
 
-        self.connect("notify::selection-mode", self._on_selection_mode_changed)
-
         action_group = Gio.SimpleActionGroup()
         actions = (
             ("play", self._on_play_action),
@@ -163,6 +155,7 @@ class AlbumWidget(Adw.Bin):
         self._album_model = self._corealbum.props.model
         self._model_signal_id = self._album_model.connect_after(
             "items-changed", self._on_model_items_changed)
+        self._disc_list_box.set_header_func(self._set_disc_header)
         self._disc_list_box.bind_model(self._album_model, self._create_widget)
 
         self._album_model.items_changed(0, 0, 0)
@@ -195,17 +188,20 @@ class AlbumWidget(Adw.Bin):
         """
         self._active_coreobject = coreobject
 
+    def _set_disc_header(
+            self, row: Gtk.ListBoxRow, before: Gtk.ListBoxRow) -> None:
+        header_label = _(f"Disc {row.props.disc_nr}")
+        header = Gtk.Label(label=header_label)
+        header.props.xalign = 0.0
+        header.props.halign = Gtk.Align.FILL
+        header.add_css_class("disc-label")
+        header.add_css_class("heading")
+        row.set_header(header)
+
     def _create_widget(self, disc: CoreDisc) -> DiscBox:
         disc_box = DiscBox(self._application, self._corealbum, disc)
         disc_box.connect('song-activated', self._song_activated)
 
-        self._corealbum.bind_property(
-            "selected", disc, "selected", GObject.BindingFlags.SYNC_CREATE)
-        self.bind_property(
-            "selection-mode", disc_box, "selection-mode",
-            GObject.BindingFlags.BIDIRECTIONAL
-            | GObject.BindingFlags.SYNC_CREATE)
-
         return disc_box
 
     def _on_model_items_changed(
@@ -214,11 +210,11 @@ class AlbumWidget(Adw.Bin):
         n_items: int = model.get_n_items()
         if n_items == 1:
             discbox = self._disc_list_box.get_row_at_index(0)
-            discbox.props.show_disc_label = False
+            discbox.get_header().props.visible = False
         else:
             for i in range(n_items):
                 discbox = self._disc_list_box.get_row_at_index(i)
-                discbox.props.show_disc_label = True
+                discbox.get_header().props.visible = True
 
         empty_album = (n_items == 0)
         self._play_button.props.sensitive = not empty_album
@@ -245,7 +241,7 @@ class AlbumWidget(Adw.Bin):
         mins_text = ngettext("{} minute", "{} minutes", mins).format(mins)
         year = self._corealbum.props.year
 
-        if year == "----":
+        if year is None:
             label = mins_text
         else:
             label = f"{year}, {mins_text}"
@@ -265,27 +261,8 @@ class AlbumWidget(Adw.Bin):
 
     def _song_activated(
             self, widget: Gtk.Widget, song_widget: SongWidget) -> None:
-        if self.props.selection_mode:
-            song_widget.props.selected = not song_widget.props.selected
-            return
-
         self._play(song_widget.props.coresong)
 
-    def select_all(self) -> None:
-        if self._album_model:
-            for coredisc in self._album_model:
-                coredisc.props.selected = True
-
-    def deselect_all(self) -> None:
-        if self._album_model:
-            for coredisc in self._album_model:
-                coredisc.props.selected = False
-
-    def _on_selection_mode_changed(
-            self, widget: Gtk.Widget, value: GObject.ParamSpecBoolean) -> None:
-        if not self.props.selection_mode:
-            self.deselect_all()
-
     def _on_add_favorites_action(
             self, action: Gio.SimpleAction,
             data: Optional[GLib.Variant]) -> None:
@@ -298,30 +275,18 @@ class AlbumWidget(Adw.Bin):
     def _on_add_playlist_action(
             self, action: Gio.SimpleAction,
             data: Optional[GLib.Variant]) -> None:
-
-        def on_response(dialog: PlaylistDialog, response_id: int) -> None:
-            if not self._playlist_dialog:
-                return
-
-            if response_id == Gtk.ResponseType.ACCEPT:
-                playlist = self._playlist_dialog.props.selected_playlist
-                coresongs = [
-                    song
-                    for disc in self._corealbum.props.model
-                    for song in disc.props.model]
-                playlist.add_songs(coresongs)
-
-            self._playlist_dialog.destroy()
-            self._playlist_dialog = None
-
         if not self._corealbum:
             return
 
-        self._playlist_dialog = PlaylistDialog(self._application)
+        selected_songs = [
+            song
+            for disc in self._corealbum.props.model
+            for song in disc.props.model]
+
+        self._playlist_dialog = PlaylistDialog(
+            self._application, selected_songs)
         active_window = self._application.props.active_window
-        self._playlist_dialog.props.transient_for = active_window
-        self._playlist_dialog.connect("response", on_response)
-        self._playlist_dialog.present()
+        self._playlist_dialog.present(active_window)
 
     def _on_play_action(
             self, action: Gio.SimpleAction,
